import ru.ifmo.testlib.Checker;
import ru.ifmo.testlib.InStream;
import ru.ifmo.testlib.Outcome;
import static ru.ifmo.testlib.Outcome.Type.*;

import java.util.HashSet;
import java.util.Arrays;

public class Check implements Checker {
    public Outcome outcome(Outcome.Type type, final String format, final Object... args) {
        return new Outcome(type, String.format(format, args));
    }

    public int compare(String a, String b) {
        for (int i = 0; i < Math.min(a.length(), b.length()); i++) {
            if (a.charAt(i) != b.charAt(i)) {
                return a.charAt(i) - b.charAt(i);
            }
        }
        return a.length() - b.length();
    }

    public boolean checkJumps(int n, int[] stones, String path) {
        char[] cpath = path.toCharArray();
        int stone = 0;
        boolean[] haveStones = new boolean[n];
        for (int i = 0; i < stones.length; i++) {
            haveStones[stones[i]] = true;
        }
        haveStones[0] = true;
        for (int i = 0; i < cpath.length; i++) {
            if (stone >= n || !haveStones[stone]) {
                return false;
            }
            if (cpath[i] == '1') {
                stone++;
            } else if (cpath[i] == '2') {
                stone += 2;
            } else {
                return false;
            }
        }
        return stone == n;
    }

    public Outcome test(InStream inf, InStream ouf, InStream ans) {
        int n = inf.nextInt();
        int k = inf.nextInt();
        int[] stones = new int[k];
        for (int i = 0; i < k; i++) {
            stones[i] = inf.nextInt();
        }
        int jLen = ans.nextInt();
        int pLen = ouf.nextInt();
        if (pLen == -1) {
            if (jLen == -1) {
                return outcome(OK, "No path");
            } else {
                return outcome(WA, "Jury found answer, but participant didn't find it");
            }
        }
        if (jLen == -1) {
            String pAns = ouf.nextToken();
            if (checkJumps(n, stones, pAns)) {
                return outcome(FAIL, "Participant found answer while jury didn't");
            }
            return outcome(WA, "Jury didn't find answer, but participant found wrong one");
        }
        if (jLen < pLen) {
            return outcome(WA, String.format("Too long path: expected length %d, but found %d", jLen, pLen));
        }
        String jAns = ans.nextToken();
        String pAns = ouf.nextToken();
        if (pAns.length() != pLen) {
            return outcome(WA, String.format("Integer in first line of output is not equal to path length: %d and %d", pLen, pAns.length()));
        }
        boolean jCheck = checkJumps(n, stones, jAns);
        boolean pCheck = checkJumps(n, stones, pAns);
        if (!jCheck) {
            return outcome(FAIL, "Jury answer is wrong");
        }
        if (!pCheck) {
            return outcome(WA, "Participant found incorrect path");
        }
        if (jLen > pLen) {
            return outcome(FAIL, String.format("Jury found wrong length - expected %d, but %d exists", jLen, pLen));
        }
        if (compare(jAns, pAns) > 0) {
            return outcome(FAIL, "Jury found non-optimal answer - participant answer is lexicographically smaller");
        }
        if (!jAns.equals(pAns)) {
            return outcome(WA, "Jury has lexicographically smaller answer");
        }
        return outcome(OK, String.format("%d jumps", jLen));
    }
}
